{******************************************************************************

 ANALOG DEVICES 
 EUROPEAN DSP APPLICATIONS
 
 versatile FLASH utility for embedded systems
  
 Command Module for dispatch,					   Ver: 0.90.0
 
	created     24-AUG-97 by hs
	last change 28-SEP-97 by hs

 Status:
 	verfied for ADmsp402/403 EB

 ******************************************************************************
}
.module/ram     COMMAND;

.include        <constant.h>;
.include        <data.h>;

.entry          lastCmdOK;
.entry          cmdProc;

.var	cmd_rev[5];
.init	cmd_rev: 0x302E, 0x3930, 0x2E30, 0x0D0A, 0x0000;

.var	temp;


{ This sections shows the logon banner and prepares the command dispacher
}
cmdProc:
	
	ax1 = 0x0c;			{ clear screen }
	call out_char_ax1;
	i7= ^stars;			{ Analog Devices Banner }
	call send_16;
	i7= ^header;			{ Analog Devices Banner }
	call send_16;
	i7= ^stars;			{ Analog Devices Banner }
	call send_16;
        ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
	i7 = ^press;			{ hint to press ?? for help }
	call send_16;
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
        i7 = ^ready;
        call send_16;


cmdProc2:
        call get_int_ar_to;		{ get the character combination }

        if lt jump cmdProc2;		{ timed out, restart }
        none = pass ar;
        if eq jump cmdProc2;		{ no character, restart }

{ Here starts the command dispatcher. All commands to react on must consist of
  two characters. The reception is timed out; this grants the correct combination
  of character to branch on
}

	m7 = 1;
	l7 = 0;
	
	ar = setbit 5 of ar;		{ remove capital letters }
	ar = setbit 13 of ar;		{ remove capital letters }

	ay0 = 0x3f3f;			{ ASCII for '??' }
	none = ar - ay0;		{ Help screen }
	if eq jump help;

	ay0 = 0x7365;			{ ASCII for 'se' }
	none = ar - ay0;		{ erase FLASH }
	if eq jump erase;

	ay0 = 0x646C;           	{ ASCII for 'dl' }
	none = ar - ay0;		{ receive data from PC }
	if eq jump download;

	ay0 = 0x6462;           	{ ASCII for 'db' }
	none = ar - ay0;		{ burn boot code to FLASH }
	if eq jump bootdown;

	ay0 = 0x6269;           	{ ASCII for 'bi' }
	none = ar - ay0;		{ display revision }
	if eq jump board_info;

	ay0 = 0x6669;           	{ ASCII for 'fi' }
	none = ar - ay0;		{ display flash usage }
	if eq jump flash_info;

	ay0 = 0x6873;           	{ ASCII for 'hs' }
	none = ar - ay0;		{ protected: update monitor }
	if eq jump update_mon;

{ when the program gets here, no correct command was regconized. The input chars are 
  echoed to the user and an error message is displayed }

lastCmdBad:

	call out_int_ar;		{ echo input }
	ar = 0x0a0d;			{ send line feed/carriage return }
	call out_int_ar;

        jump cmdProc;			{ return for new chars }

lastCmdOK:
	jump cmdProc2;

{******************************************************************************
 *
 *  FUNCTION: HELP MENU
 *
 *  received: ??
 *
 ******************************************************************************}
help:
	ax1 = 0x0c;
	call out_char_ax1;		{ clear screen }

	i7 = ^stars;			{ send help banner }
	call send_16;
	i7 = ^help_h;
	call send_16;
	i7 = ^stars;
	call send_16;

	i7 = ^help_s;			{ send help page }
	call send_16;
	
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;

        jump cmdProc;


{******************************************************************************
 *
 *  FUNCTION: board information and revisions
 *
 *  received: bi
 *
 ******************************************************************************}
board_info:
	ax1 = 0x0c;
	call out_char_ax1;		{ clear screen }

	i7 = ^stars;			{ send board banner }
	call send_16;
	i7 = ^board_h;
	call send_16;
	i7 = ^stars;
	call send_16;

	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;

	i7 = ^mod_rev;
	call send_16;

	i7 = ^cmd_nam;
	call send_16;
	i7 = ^cmd_rev;
	call send_16;

	i7 = ^mot_nam;
	call send_16;
	i7 = ^mot_rev;
	call send_16;

	i7 = ^fls_nam;
	call send_16;
	i7 = ^fls_rev;
	call send_16;

	i7 = ^uat_nam;
	call send_16;
	i7 = ^uat_rev;
	call send_16;

	i7 = ^dta_nam;
	call send_16;
	i7 = ^dta_rev;
	call send_16;

	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;

        jump cmdProc;


{******************************************************************************
 *
 *  FUNCTION: Information on flash usage
 *
 *  received: fi
 *
 ******************************************************************************}
flash_info:
	ax1 = 0x0c;
	call out_char_ax1;		{ clear screen }

	i7 = ^stars;			{ send erase banner }
	call send_16;
	i7 = ^erase_h;
	call send_16;
	i7 = ^stars;
	call send_16;

	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;

	ar = 0x0;			{ sector 0 = 16kbyte 0x00000 - 0x03FFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	ax1 = 0x30;
	dm(temp) = ax1;
	call fi0;

	ar = 0x4000;			{ sector 1 =  8kbyte 0x04000 - 0x05FFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	ax1 = 0x31;
	dm(temp) = ax1;
	call fi0;
	
	ar = 0x6000;			{ sector 2 =  8kbyte 0x06000 - 0x07FFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	ax1 = 0x32;
	dm(temp) = ax1;
	call fi0;

	ar = 0x8000;			{ sector 3 = 32kbyte 0x08000 - 0x0FFFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	ax1 = 0x33;
	dm(temp) = ax1;
	call fi0;

	ar = 0x0000;			{ sector 4 = 64kbyte 0x10000 - 0x1FFFF }
	dm(addr_lo) = ar;
	ar = 0x01;
	dm(addr_hi) = ar;
	ax1 = 0x34;
	dm(temp) = ax1;
	call fi0;

	ar = 0x0000;			{ sector 5 = 64kbyte 0x20000 - 0x2FFFF }
	dm(addr_lo) = ar;
	ar = 0x02;
	dm(addr_hi) = ar;
	ax1 = 0x35;
	dm(temp) = ax1;
	call fi0;

	ar = 0x0000;			{ sector 6 = 64kbyte 0x30000 - 0x3FFFF }
	dm(addr_lo) = ar;
	ar = 0x03;
	dm(addr_hi) = ar;
	ax1 = 0x36;
	dm(temp) = ax1;
	call fi0;

	ar = 0x0000;			{ sector 7 = 64kbyte 0x40000 - 0x4FFFF }
	dm(addr_lo) = ar;
	ar = 0x04;
	dm(addr_hi) = ar;
	ax1 = 0x37;
	dm(temp) = ax1;
	call fi0;

	ar = 0x0000;			{ sector 8 = 64kbyte 0x50000 - 0x5FFFF }
	dm(addr_lo) = ar;
	ar = 0x05;
	dm(addr_hi) = ar;
	ax1 = 0x38;
	dm(temp) = ax1;
	call fi0;

	ar = 0x0000;			{ sector 9 = 64kbyte 0x60000 - 0x6FFFF }
	dm(addr_lo) = ar;
	ar = 0x06;
	dm(addr_hi) = ar;
	ax1 = 0x39;
	dm(temp) = ax1;
	call fi0;
	
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;

	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;
	jump cmdProc;
	
fi0:	i7 = ^the_sect;
	call send_16;
	ax1 = dm(temp);
	call out_char_ax1;

	call read_byte;
	ay0 = 0xff;
	ar = dm(d_byte);
	ar = ar - ay0;
	if eq jump fi2;
	
fi1:	i7 = ^contains;
	call send_16;
	rts;
	
fi2:	i7 = ^be_empty;
	call send_16;
	rts;


{******************************************************************************
 *
 *  FUNCTION: Chip erase or single blocks
 *
 *  received: se
 *
 ******************************************************************************}
erase:
	ax1 = 0x0c;
	call out_char_ax1;		{ clear screen }

	i7 = ^stars;			{ send erase banner }
	call send_16;
	i7 = ^erase_h;
	call send_16;
	i7 = ^stars;
	call send_16;

	ar = 0x80;
	dm(c_byte) = ar;

	i7 = ^sector;
	call send_16;
	call get_char_ax1;		{ inkey page }
	call out_char_ax1;
	ay0 = 0x30;
	ar = ax1 - ay0;			{ offset to number }
	if lt jump cmdProc;		{ no number }
	ay0 = 0xa;
	af = ar - ay0;
	if ge jump cmdProc;		{ no number }
	
	ay0 = ^s_jtab;			{ pic the right page }
	ar = ar + ay0;
	i7 = ar;
	jump (i7);
	
s_jtab:
	jump sect0;
	jump sect1;
	jump sect2;
	jump sect3;
	jump sect4;
	jump sect5;
	jump sect6;
	jump sect7;
	jump sect8;
	jump sect9;
	
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;

        jump erase;

sect0:
	ar = 0x0;			{ sector 0 = 16kbyte 0x00000 - 0x03FFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	jump se_end;

sect1:
	ar = 0x4000;			{ sector 1 =  8kbyte 0x04000 - 0x05FFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	jump se_end;
	
sect2:
	ar = 0x6000;			{ sector 2 =  8kbyte 0x06000 - 0x07FFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	jump se_end;

sect3:
	ar = 0x8000;			{ sector 3 = 32kbyte 0x08000 - 0x0FFFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;
	jump se_end;

sect4:
	ar = 0x0000;			{ sector 4 = 64kbyte 0x10000 - 0x1FFFF }
	dm(addr_lo) = ar;
	ar = 0x01;
	dm(addr_hi) = ar;
	jump se_end;

sect5:
	ar = 0x0000;			{ sector 5 = 64kbyte 0x20000 - 0x2FFFF }
	dm(addr_lo) = ar;
	ar = 0x02;
	dm(addr_hi) = ar;
	jump se_end;

sect6:
	ar = 0x0000;			{ sector 6 = 64kbyte 0x30000 - 0x3FFFF }
	dm(addr_lo) = ar;
	ar = 0x03;
	dm(addr_hi) = ar;
	jump se_end;

sect7:
	ar = 0x0000;			{ sector 7 = 64kbyte 0x40000 - 0x4FFFF }
	dm(addr_lo) = ar;
	ar = 0x04;
	dm(addr_hi) = ar;
	jump se_end;

sect8:
	ar = 0x0000;			{ sector 8 = 64kbyte 0x50000 - 0x5FFFF }
	dm(addr_lo) = ar;
	ar = 0x05;
	dm(addr_hi) = ar;
	jump se_end;

sect9:
	ar = 0x0000;			{ sector 9 = 64kbyte 0x60000 - 0x6FFFF }
	dm(addr_lo) = ar;
	ar = 0x06;
	dm(addr_hi) = ar;
	jump se_end;
	
se_end:
	call sect_erase;

	i7= ^done;			{ job finished }
	call send_16;
	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;
	jump erase;			{ exit by not pressing a number }


{******************************************************************************
 *
 *  FUNCTION: download application to FLASH
 *
 *  received: dl
 *
 ******************************************************************************}
download:
	ax1 = 0x0c;
	call out_char_ax1;		{ clear screen }

	i7 = ^stars;			{ send erase banner }
	call send_16;
	i7 = ^prog_h;
	call send_16;
	i7 = ^stars;
	call send_16;

	i7 = ^abort;			{ hit <ESC> to abort }
	call send_16;

	i7 = ^sec_ent;			{ sector to download ? }
	call send_16;
	
	call get_char_ax1;		{ get sector }
	call out_char_ax1;
	ay0 = 0x30;			{ no number }
	ar = ax1 - ay0;			{ offset to number }
	if lt jump cmdProc;		{ no number }
	ay0 = 0x6;			{ not valid >=6 }
	af = ar - ay0;
	if gt jump download;
	dm(offset) = ar;		{ download to desired 64k block }

	i7 = ^waiting;			{ waiting for file }
	call send_16;

	ar = 0x0;			{ 0 for user download }
	dm(mflag) = ar;
	
	ar = 0xA0;			{ program byte }
	dm(c_byte) = ar;

	call mot_read;			{ read the BNM file }	

	i7 = ^done;
	call send_16;

	i7 = ^rem_boot;
	call send_16;

	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;

        jump cmdProc;

{******************************************************************************
 *
 *  FUNCTION: download modified boot code
 *
 *  received: db
 *
 ******************************************************************************}
bootdown:
	ax1 = 0x0c;
	call out_char_ax1;		{ clear screen }

	i7 = ^stars;			{ send boot banner }
	call send_16;
	i7 = ^erase_h;
	call send_16;
	i7 = ^stars;
	call send_16;

	i7 = ^boot_dld;
	call send_16;

	i0 = ^bootcode;
	m0 = 1;
	l0 = 0;

	ar = 0x0;			{ sector 0 = 16kbyte 0x00000 - 0x03FFF }
	dm(addr_lo) = ar;
	ar = 0x0;
	dm(addr_hi) = ar;

	ar = 0xA0;			{ program byte }
	dm(c_byte) = ar;
	
	cntr = %bootcode;
	do bw_1 until ce;
		ar = dm(i0,m0);
		dm(d_byte) = ar;
		call prog_byte;
		call auto_inc;
	bw_1:	nop;	
	
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
        
        i7 = ^done;
        call send_16;
	
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;

	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;

        jump cmdProc;


{******************************************************************************
 *
 *  FUNCTION: Update Monitor (protected)
 *
 *  received: hs
 *
 ******************************************************************************}
update_mon:

	ax1 = 0x0c;
	call out_char_ax1;		{ clear screen }

	i7 = ^stars;			{ send erase banner }
	call send_16;
	i7 = ^update_h;
	call send_16;
	i7 = ^stars;
	call send_16;
	
	i7 = ^proceed;			{ do you really want to update ? }
	call send_16;
	idle;
	call get_char_ax1;
	call out_char_ax1;

	ay0 = 0x79;			{ check for 'y' }
	ar = ax1 - ay0;
	if eq jump yes_upd;
	jump no_upd;

yes_upd:
	ar = 0x80;			{ erase sector }
	dm(c_byte) = ar;

	ar = 0x0000;			{ sector 10 = 64kbyte 0x70000 - 0x7FFFF }
	dm(addr_lo) = ar;
	ar = 0x07;
	dm(addr_hi) = ar;
	call sect_erase;

	ar = 0x1;			{ update monitor }
	dm(mflag) = ar;

	ar = 0xA0;			{ program byte }
	dm(c_byte) = ar;

	ar = 0x07;
	dm(offset) = ar;		{ download to 0x70000-0x7ffff }

	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;

	i7 = ^waiting;			{ waiting for file }
	call send_16;

	call mot_read;			{ read the BNM file }	

	i7 = ^done;
	call send_16;

no_upd:
	ar = 0x0a0d;			{ send line feed/carriage return }
        call out_int_ar;
	i7 = ^enter;			{ wait for enter }
	call send_16;
	call wait_cr;

        jump cmdProc;

.endmod;
