{******************************************************************************

 ANALOG DEVICES 
 EUROPEAN DSP APPLICATIONS
 
 versatile FLASH utility for embedded systems
  
 flash algorithm module,					   Ver: 1.00.0
 
	created     15-SEP-97 by hs
	last change 28-SEP-97 by hs

 Status:
 	verfied for ADmsp402/403 EB

 ******************************************************************************
}
.module/ram flash_srv;

.include    <constant.h>;

.var	fls_rev[5];
.global fls_rev;
.init	fls_rev: 0x312E, 0x3030, 0x2E30, 0x0D0A, 0x0000;

.var/dm/ram	d_byte;			{ holds value read / written }
.global		d_byte;
.var/dm/ram	d_btmp;			{ temporaray value read / written }

.var/dm/ram	c_byte;			{ flash command }
.global		c_byte;

.var/dm/ram	addr_hi;		{ upper  8bit of 22bit address }
.global		addr_hi;
.var/dm/ram	addr_ht;		{ upper  8bit of BDMA address }

.var/dm/ram	addr_lo;		{ lower 16bit of 22bit address }
.global		addr_lo;
.var/dm/ram	addr_lt;		{ lower 16bit of BDMA address }

.entry	bdma_setup;
.entry	prog_byte;
.entry	read_byte;
.entry	sect_erase;
.entry  auto_inc;

{******************************************************************************
 *
 *  Global setup for the BDMA port
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : none
 *  modify : PFTYPE, BDMA_BIAD
 *  output : none
 *  destroy: ar, ax0, ay0
 *  calls  : none
 *
 ******************************************************************************}
bdma_setup:
	ax0 = dm(PFTYPE);
	ay0 = 0x7000;			{ 7 wait states for BDMA access}
	ar = ax0 or ay0;
	dm(PFTYPE) = ar;
	
	ar = imask;
	ar = setbit 3 of ar;
	imask = ar;

	rts;

{******************************************************************************
 *
 *  Flash application server
 *
 *    byte program:
 *
 *	c_byte : 0xA0
 *	d_byte : value
 *	addr_lo: low 16bit
 *	addr_hi: high 6bit
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : see above
 *  modify : addr_lt, addr_ht, d_btmp
 *  output : see above
 *  destroy: ar, ax0, ay0, af
 *  calls  : none
 *
 ******************************************************************************}
prog_byte:
	call init_seq;
	call cmd_write;
	
	ar = dm(d_byte);
	dm(d_btmp) = ar;
	call calc_adr;
	
pb_1:	ar = ^d_btmp;			{ readback command }
	dm(BDMA_BIAD) = ar;
	ar = dm(addr_lt);
	dm(BDMA_BEAD) = ar;		{ write again lower 14 bit }
	ar = dm(addr_ht);
	dm(BDMA_BDMA_Ctrl) = ar;	{ write higher 8 bit & control }
	ar = 0x1;
	dm(BDMA_BWCOUNT) = ar;
	
pb_2:	idle;				{ wait for BDMA access completed }

	ay0 = 0x80;			{ comparison sequence }
	ax0 = dm(d_byte);
	af = ax0 and ay0;		{ af holds bit 7 of d_byte }
	ax0 = dm(d_btmp);
	ar = ax0 and ay0;		{ ar holds bit 7 of readback }
	ar = ar - af;
	
	if ne jump pb_1;		{ not sucessfully programmed reread }

	rts;

{******************************************************************************
 *
 *  Flash application server
 *
 *    byte readback:
 *
 *	c_byte : don't care
 *	d_byte : readback value
 *	addr_lo: low 16bit
 *	addr_hi: high 6bit
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : see above
 *  modify : addr_lt, addr_ht, d_btmp, d_byte
 *  output : see above
 *  destroy: ar
 *  calls  : none
 *
 ******************************************************************************}
read_byte:
	ar = ^d_byte;
	dm(BDMA_BIAD) = ar;
	i7 = dm(addr_lo);		{ set the lower 14 bit }
	dm(BDMA_BEAD) = i7;
	dm(addr_lt) = i7;		{ store low address for readback }
	sr1 = dm(addr_hi);
	sr = lshift sr1 by 10 (hi);
	ar = dm(addr_lo);
	sr = sr or lshift ar by 10 (lo);
	ay0 = 0xff00;
	ar = sr1 and ay0;		{ holds now the upper 8 page bits }
	ay0 = 0x3;
	ar = ar or ay0;
	dm(BDMA_BDMA_Ctrl) = ar;
	ar = 0x1;
	dm(BDMA_BWCOUNT) = ar;

	idle;				{ wait for BDMA access completed }
	rts;

{******************************************************************************
 *
 *  Flash application server
 *
 *    sector erase:
 *
 *	c_byte : 0x80
 *	d_byte : don't care
 *	addr_lo: low 16bit
 *	addr_hi: high 6bit
 *
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : see above
 *  modify : addr_lt, addr_ht, d_btmp
 *  output : see above
 *  destroy: ax0, ay0, ar, af, sr0, sr1
 *  calls  : none
 *
 ******************************************************************************}
sect_erase:
	call init_seq;
	call cmd_write;
	call init_seq;
		
	ar = 0x30;
	dm(d_btmp) = ar;		{ sector erase command 0x30 }
	call calc_adr;
	
se_1:	ar = ^d_btmp;			{ readback command }
	dm(BDMA_BIAD) = ar;
	ar = dm(addr_lt);
	dm(BDMA_BEAD) = ar;		{ write again lower 14 bit }
	ar = dm(addr_ht);
	dm(BDMA_BDMA_Ctrl) = ar;	{ write higher 8 bit & control }
	ar = 0x1;
	dm(BDMA_BWCOUNT) = ar;

se_2:	idle;				{ wait for BDMA access completed }

	ar = dm(d_btmp);		{ comparison sequence }
	ar = tstbit 5 of ar;
	if ne jump fs_error;
	ar = dm(d_btmp);
	ar = tstbit 7 of ar;
	if eq jump se_1;		{ still in erase, reread }
		
	rts;


{******************************************************************************
 *
 *  Flash application server subroutine
 *  start FLASH embedded algorithms
 *
 ******************************************************************************}
init_seq:
	ar = 0xaa;			{ start sequence, first command }
	dm(d_btmp) = ar;
	ar = ^d_btmp;
	dm(BDMA_BIAD) = ar;
	ar = 0x555;
	dm(BDMA_BEAD) = ar;
	ar = 0x7;
	dm(BDMA_BDMA_Ctrl) = ar;
	ar = 0x1;
	dm(BDMA_BWCOUNT) = ar;

fs_1:	idle;				{ wait for BDMA access completed }

	ar = 0x55;			{ second command }
	dm(d_btmp) = ar;
	ar = ^d_btmp;
	dm(BDMA_BIAD) = ar;
	ar = 0x2aa;
	dm(BDMA_BEAD) = ar;
	ar = 0x7;
	dm(BDMA_BDMA_Ctrl) = ar;
	ar = 0x1;
	dm(BDMA_BWCOUNT) = ar;

fs_2:	idle;				{ wait for BDMA access completed }
	rts;
	
{******************************************************************************
 *
 *  Flash application server subroutine
 *  do the command word write
 *
 ******************************************************************************}
cmd_write:
	ar = dm(c_byte);		{ third command }
	dm(d_btmp) = ar;		{ now get the command byte }
	ar = ^d_btmp;
	dm(BDMA_BIAD) = ar;
	ar = 0x555;
	dm(BDMA_BEAD) = ar;
	ar = 0x7;
	dm(BDMA_BDMA_Ctrl) = ar;
	ar = 0x1;
	dm(BDMA_BWCOUNT) = ar;

fs_3:	idle;				{ wait for BDMA access completed }
	rts;
	
{******************************************************************************
 *
 *  Flash application server subroutine
 *  split the 22bit address
 *
 ******************************************************************************}
calc_adr:
	ar = ^d_btmp;
	dm(BDMA_BIAD) = ar;
	i7 = dm(addr_lo);		{ set the lower 14 bit }
	dm(BDMA_BEAD) = i7;
	dm(addr_lt) = i7;		{ store low address for readback }
	sr1 = dm(addr_hi);
	sr = lshift sr1 by 10 (hi);
	ar = dm(addr_lo);
	sr = sr or lshift ar by 10 (lo);
	ay0 = 0xff00;
	ar = sr1 and ay0;		{ holds now the upper 8 page bits }
	ay0 = 0x7;
	ar = ar or ay0;
	dm(BDMA_BDMA_Ctrl) = ar;
	ar = clrbit 2 of ar;
	dm(addr_ht) = ar;		{ store high address & control for readback }
	ar = 0x1;
	dm(BDMA_BWCOUNT) = ar;

fs_4:	idle;				{ wait for BDMA access completed }
	rts;

{******************************************************************************
 *
 *  Flash application server subroutine
 *  error treatment
 *
 ******************************************************************************}
fs_error:
	ar = 0xff;
	dm(c_byte) = ar;
	rts;


{******************************************************************************
 *
 *  autoincrement external address;
 *
 *  REGISTER USAGE SUMMARY:
 *
 *  input  : none
 *  modify : addr_lo, addr_hi
 *  output : none
 *  destroy: ar
 *  calls  : none
 *
 ******************************************************************************}
auto_inc:
	ar = dm(addr_lo);
	ar = ar + 1;
	dm(addr_lo) = ar;
	if not AC rts;
	
	ar = dm(addr_hi);
	ar = ar + 1;
	dm(addr_hi) = ar;
	rts;	

.endmod;
